@@ -28,7 +28,7 @@
 # Version
 # -------
 #
-# * tffpy version = 0.1.3
+# * tffpy version = 0.1.4
 #
 # Licence
 # -------
@@ -64,6 +64,7 @@ import matplotlib.pyplot as plt
 import matplotlib
 import pandas as pd
 from pathlib import Path
+from scipy.stats import linregress
 
 from yafe import Experiment
 from madarrays import Waveform
@@ -114,7 +115,7 @@ class SolveTffExperiment(Experiment):
         -------
         int
         """
-        return len(list((self.xp_path / 'tasks').glob('*')))
+        return len(list((self.xp_path / 'tasks').glob('0*')))
 
     @staticmethod
     def get_experiment(setting='full', force_reset=False):
@@ -154,7 +155,8 @@ class SolveTffExperiment(Experiment):
                               fig_dir=None)
         solver_params = dict(tol_subregions=[None, 1e-5],
                              tolerance_arrf=1e-3,
-                             proba_arrf=1 - 1e-4)
+                             proba_arrf=1 - 1e-4,
+                             rand_state=0)
         if setting == 'light':
             data_params['loc_source'] = 'bird'
             data_params['wideband_src'] = 'car'
@@ -262,6 +264,8 @@ class SolveTffExperiment(Experiment):
 
         # Scatter plot for running times : tff-1 vs. tff-P
         plt.figure()
+        x = []
+        y = []
         for win_type in coords_dict['problem_win_choice']['data']:
             t_tff1 = results.sel(measure=['t_lambda_tff', 't_arrf', 't_evdn'],
                                  problem_win_choice=win_type,
@@ -281,9 +285,32 @@ class SolveTffExperiment(Experiment):
                 win_type_label = 'Gauss'
             else:
                 win_type_label = 'Hann'
+            slope, intercept, r_value, p_value, std_err = \
+                linregress(t_tff1.values.reshape(-1), t_tffp.values.reshape(-1))
+            print('Running times ({}): slope={}, intercept={}, 1/slope={},'
+                  .format(win_type, slope, intercept, 1 / slope))
             plt.plot(t_tff1.values.reshape(-1),
                      t_tffp.values.reshape(-1),
                      '+', label=win_type_label)
+            # plt.plot(t_tff1.values.reshape(-1),
+            #          slope * t_tff1.values.reshape(-1) + intercept)
+            x.append(t_tff1.values.reshape(-1).copy())
+            y.append(t_tffp.values.reshape(-1).copy())
+
+        x = np.array(x)
+        y = np.array(y)
+        # for i in range(x.shape[1]):
+        #     plt.plot(x[:, i], y[:, i], ':k')
+
+        x = x.reshape(-1)
+        y = y.reshape(-1)
+        slope, intercept, r_value, p_value, std_err = linregress(x, y)
+        print('Running times (all): slope={}, intercept={}, 1/slope={},'
+              .format(slope, intercept, 1 / slope))
+        print('Linear slope (not affine):')
+        print(np.vdot(x, y) / np.vdot(x, x), np.vdot(x, x) / np.vdot(x, y))
+
+        # plt.plot(x, slope * x + intercept)
         plt.xlabel(r'Running time for TFF-1 (s)')
         plt.ylabel(r'Running time for TFF-P (s)')
         plt.legend()
@@ -614,7 +641,7 @@ class SolveTffExperiment(Experiment):
         plot_spectrogram(x=x_wb, dgt_params=dgt_params,
                          fs=signal_params['fs'],
                          clim=clim)
-        plt.title('True source')
+        plt.title('True signal')
         plt.tight_layout()
         plt.savefig(fig_dir / 'spectrogram_true_wb_source.pdf')
 
@@ -776,10 +803,12 @@ class Solver:
         :py:meth:`~tffpy.tf_fading.GabMulTff.compute_decomposition`.
     """
 
-    def __init__(self, tol_subregions, tolerance_arrf, proba_arrf):
+    def __init__(self, tol_subregions, tolerance_arrf, proba_arrf,
+                 rand_state=0):
         self.tol_subregions = tol_subregions
         self.tolerance_arrf = tolerance_arrf
         self.proba_arrf = proba_arrf
+        self.rand_state = rand_state
 
     def __call__(self, x_mix, mask, dgt_params, signal_params):
         """
@@ -824,10 +853,11 @@ class Solver:
             The estimated solution and additional information
         """
         gmtff = GabMulTff(x_mix=x_mix, mask=mask, dgt_params=dgt_params,
-                        signal_params=signal_params,
-                        tol_subregions=self.tol_subregions)
+                          signal_params=signal_params,
+                          tol_subregions=self.tol_subregions)
         gmtff.compute_decomposition(tolerance_arrf=self.tolerance_arrf,
-                                  proba_arrf=self.proba_arrf)
+                                    proba_arrf=self.proba_arrf,
+                                    rand_state=self.rand_state)
 
         # Estimate energy and lambda
         lambda_tff, t_lambda_tff = gmtff.compute_lambda(x_mix=x_mix)
@@ -868,8 +898,9 @@ def perf_measures(task_params, source_data, problem_data,
     -------
     dict
         All data useful for result analysis including SDR and Itakura-Saito
-        performance, running times, hyperparameter values, mask size and
-        number of sub-regions.
+        performance, running times, hyperparameter values, mask size,
+        number of sub-regions, estimated rank (summed over sub-regions),
+        lowest singular value.
     """
     x_tff = solved_data['x_tff']
     x_zero = solved_data['x_zero']
@@ -923,7 +954,11 @@ def perf_measures(task_params, source_data, problem_data,
                          t_subreg=solved_data['t_subreg']
                          )
     features = dict(mask_size=np.sum(gmtff.mask > 0),
-                    n_subregions=gmtff.n_areas)
+                    mask_ratio=np.mean(gmtff.mask > 0),
+                    n_subregions=gmtff.n_areas,
+                    rank_sum=np.sum([s.size for s in gmtff.s_vec_list]),
+                    lowest_sv=np.min([np.min(s) for s in gmtff.s_vec_list])
+                    )
     return dict(**running_times, **sdr_res, **is_res, **features)
 
 
